
#include <stdio.h>
#include <model.h>
#include <carver.h>

static Linklist tree;

Linklist rawlist;

Object *curr_obj = NULL;
Method *curr_met = NULL;

void open_class(char *name) {
	fprintf(stderr,"CLASS: %s\n",name);

	// create the new object
	if(curr_obj) close_class();
	curr_obj = new Object((string)name);
}

void add_inheritance(char *name) {
	fprintf(stderr,"CLASS INHERITS FROM %s\n",name);
	if(!curr_obj) {
		fprintf(stderr,"CAN'T INHERIT: NO CLASS OPEN\n");
		return;
	}
	curr_obj->parent = new Object(name);
}

void close_class() {
	if(!curr_obj) {
		fprintf(stderr,"CAN'T CLOSE CLASS: NONE OPEN\n");
		return;
	}

	if( curr_met ) close_method();

	if( curr_obj->methods.length == 0
	 && curr_obj->properties.length == 0) {
		// discard void class
		delete curr_obj;
		curr_obj = NULL;
		fprintf(stderr,"CLASS DISCARDED\n\n");
	} else {
		// place it in the unsorted list
		rawlist.append( curr_obj );
		curr_obj = NULL;
		fprintf(stderr,"CLASS INCLUDED\n\n");
	}
}

void open_method(char *name) {
	if(curr_met) close_method();

	curr_met = new Method( name );
	fprintf(stderr,"\tMETHOD: %s\n",name);
}

void add_argument(char *args) {
	Argument *new_arg;
	if( !curr_obj || !curr_met) return;

	new_arg = new Argument();
	new_arg->parse(args);
	new_arg->method = curr_met;
	curr_met->args.append( new_arg );
}

void close_method() {
	if(!curr_met || !curr_obj) return;

	curr_obj->methods.append( curr_met );
	curr_met->parent = curr_obj;
	curr_met = NULL;
}

void add_property(char *name) {
	Property *new_prop;

	if( !curr_obj ) return;

	new_prop = new Property();
	new_prop->parse(name);
	new_prop->parent = curr_obj;
	curr_obj->properties.append( new_prop ); 

	fprintf(stderr,"\tPROPERTY: %s\n",name);
}


////////////////////////////////////////////////////////
// parses all the rawlist in a tree based on inheritance
Linklist *treesort() {
	Object *e, *etmp;
	Object *s;

	if(!rawlist.length) return NULL;

	// put parents in place
	e = (Object*)rawlist.first;
	while(e) {
		if( !e->parent ) {
			etmp = (Object*)e->next;
			e->rem();
			tree.append( e );
			e = etmp;
		} else e = (Object*)e->next;
	}	

	// parse childs left and puts them in place
	e = (Object*) rawlist.first;
	while(e) {
		// search for parents
		s = (Object*) tree.first;
		while(s) {
			if(s->name == e->parent->name) {
				// parent found
				etmp = (Object*)e->next;
				e->rem();
				s->childs.append( e );
				delete e->parent;
				e->parent = s;
				e = etmp;
				break;
			}
			s = (Object*)s->next;
		}

		e = (Object*)e->next;

		if(!e) e = (Object*)rawlist.first;
	}

	return &tree;
}

